package com.hjy.modlue.news;

import com.hjy.common.annotation.Log;
import com.hjy.common.core.controller.BaseController;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.enums.BusinessType;
import com.hjy.common.enums.CbNewsType;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.module.mapper.news.CbNewsMapper;
import com.hjy.module.service.news.ICbNewsService;
import com.hjy.module.service.portal.ICbSensitiveService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.news.CbNewsVo;
import com.hjy.module.vo.portal.CbSensitiveVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

import static com.hjy.modlue.portal.CbSensitiveController.getAjaxResult;

/**
 * 网站新闻Controller
 *
 * @author Tellsea
 * @date 2023-08-22
 */
@Api(tags = "网站新闻Controller")
@RestController
@RequestMapping("/news")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CbNewsController extends BaseController {
    private final ICbNewsService cbNewsService;

    private final CbNewsMapper cbNewsMapper;

    private final IWfInstActionTaskService wfInstActionTaskService;

    private final ICbSensitiveService cbSensitiveService;


    @ApiOperation("查询网站新闻列表")
    @PreAuthorize("@ss.hasPermi('business:NEWS:list')")
    @GetMapping("/list")
    public TableDataInfo<CbNewsVo> list(CbNewsVo entity) {
        return cbNewsService.queryList(entity);
    }

    @ApiOperation("获取网站新闻详细信息")
    @PreAuthorize("@ss.hasPermi('business:NEWS:query')")
    @GetMapping(value = "/getInfo/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success("查询成功", cbNewsService.queryById(id));
    }

    @ApiOperation("新增网站新闻")
    @PreAuthorize("@ss.hasPermi('business:NEWS:add')")
    @Log(title = "网站新闻", businessType = BusinessType.INSERT)
    @PostMapping("add")
    public AjaxResult add(@RequestBody CbNewsVo entity) {
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        // 获取敏感集合
        List<CbSensitiveVo> list = cbSensitiveService.queryAll(new CbSensitiveVo());
        AjaxResult trie = getAjaxResult(entity.getContent(),list);
        if (trie != null) return trie;
        cbNewsService.save(entity);
        if (CbNewsType.AUDIT.getCode().equals(entity.getStatus())) {
            wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.NEWSSERVICE.getService(), entity.getId(), entity.getTitle());
            return AjaxResult.success("审核发起");
        }
        return AjaxResult.success("保存成功");
    }

//    @ApiOperation("网站新闻确认")
//    @PreAuthorize("@ss.hasPermi('business:NEWS:confirm')")
//    @Log(title = "网站新闻确认", businessType = BusinessType.UPDATE)
//    @PostMapping("confirm/{id}")
//    public AjaxResult confirm(@PathVariable String id) {
//        CbNews cbNews = cbNewsService.getById(id);
//        if (ObjectUtils.isEmpty(cbNews)) return error("数据不存在");
//        if (!cbNews.getStatus().equals(CbNewsType.DRAFTS.getCode())) return error("当前数据非草稿，无法确认！");
//        cbNews.setStatus(CbNewsType.CONFIRM.getCode());
//        return toAjax(cbNewsService.updateById(cbNews));
//    }


    @ApiOperation("修改网站新闻")
    @PreAuthorize("@ss.hasPermi('business:NEWS:edit')")
    @Log(title = "网站新闻", businessType = BusinessType.UPDATE)
    @PostMapping("edit")
    public AjaxResult edit(@RequestBody CbNewsVo entity) {
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        if (CbNewsType.ISSUE.getCode().equals(entity.getStatus())) {
            entity.setReleaseDate(LocalDateTime.now().toString());
        }
        return toAjax(cbNewsService.updateById(entity));
    }

    @ApiOperation("新闻置顶接口")
    @PreAuthorize("@ss.hasPermi('business:NEWS:top')")
    @Log(title = "新闻网站", businessType = BusinessType.UPDATE)
    @PostMapping("top")
    public AjaxResult top(@RequestBody CbNewsVo entity) {
        if (Objects.isNull(entity.getPicture())) {
            return AjaxResult.error("置顶图片为空,请修改");
        }
        Integer sum = cbNewsMapper.selectTop();
        if (CbNewsType.TOP.getCode().equals(entity.getStatus()) && sum >= 6) {
            return AjaxResult.error("置顶已经到达上限");
        }
        CbNewsVo cbNewsVo = new CbNewsVo();
        cbNewsVo.setStatus(entity.getStatus());
        cbNewsVo.setId(entity.getId());
        return toAjax(cbNewsService.updateById(cbNewsVo));
    }

    @ApiOperation("修改提交审核接口")
    @PreAuthorize("@ss.hasPermi('business:NEWS:audit')")
    @Log(title = "提交审核", businessType = BusinessType.UPDATE)
    @PostMapping("/audit")
    public AjaxResult audit(@RequestBody CbNewsVo entity) {
        entity.setContent(entity.getContent().replace("/dev-api", "").replace("/hjy-api", ""));
        // 获取敏感集合
        List<CbSensitiveVo> list = cbSensitiveService.queryAll(new CbSensitiveVo());
        AjaxResult trie = getAjaxResult(entity.getContent(),list);
        if (trie != null) return trie;
        cbNewsService.updateById(entity);
        wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.MODIFYNEWSSERVICE.getService(), entity.getId(), entity.getTitle());
        return AjaxResult.success("审核发起");
    }

    @ApiOperation("删除新闻管理")
    @PreAuthorize("@ss.hasPermi('business:NEWS:remove')")
    @Log(title = "删除新闻管理", businessType = BusinessType.DELETE)
    @GetMapping("/remove/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        for (String id : ids) {
            wfInstActionTaskService.delete(id,WorkFlowServiceEnums.NEWSSERVICE.getService());
            wfInstActionTaskService.delete(id,WorkFlowServiceEnums.MODIFYNEWSSERVICE.getService());
        }
        return toAjax(cbNewsService.removeByIds(Arrays.asList(ids)) ? 1 : 0);
    }

}
