package com.hjy.modlue.portalAPI;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.hjy.business.appuser.utils.JsonUtils;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.AjaxResult;
import com.hjy.common.core.domain.entity.CbCompanyUser;
import com.hjy.common.core.domain.model.LoginBody;
import com.hjy.common.core.domain.model.RegisterBody;
import com.hjy.common.core.redis.RedisCache;
import com.hjy.common.core.sm2.Sm2;
import com.hjy.common.enums.WorkFlowServiceEnums;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.*;
import com.hjy.common.utils.ip.IpUtils;
import com.hjy.common.utils.uuid.IdUtils;
import com.hjy.framework.web.service.SysLoginService;
import com.hjy.framework.web.service.TokenService;
import com.hjy.module.domain.portal.CbModifyCompanyUser;
import com.hjy.module.service.portal.ICbCompanyUserService;
import com.hjy.module.service.portal.ICbModifyCompanyUserService;
import com.hjy.module.service.wf.IWfInstActionTaskService;
import com.hjy.module.vo.portal.CbCompanyUserVo;
import com.hjy.web.controller.common.CaptchaUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.bouncycastle.crypto.InvalidCipherTextException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

/**
 * @author 12635
 */
@Api(tags = "门户登录相关接口")
@RestController
@RequestMapping("/portal_api/login")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class PortalLoginController {
    /**
     * 最大登陆次数
     */
    private static final int MAX_LOGIN_ATTEMPTS = 5;
    /**
     * 连续超过最大登陆次数的次数，触发封禁
     */
    private static final int BAN_THRESHOLD = 3;
    /**
     * 封禁时间，单位分钟
     */
    private static final Long BAN_TIME = 10L;
    /**
     * 专家类型
     */
    private static final String SPECIALIST = "2";
    /**
     * 私钥
     */
    private String privateKeyHex = null;

    private final CaptchaUtils captchaUtils;

    private final SysLoginService sysLoginService;

    private final ICbCompanyUserService cbCompanyUserService;

    private final TokenService tokenService;

    private final RedisCache redisCache;

    private final RedisUtil redisUtil;

    private final IWfInstActionTaskService wfInstActionTaskService;

    private final ICbModifyCompanyUserService cbModifyCompanyUserService;


    @ApiOperation("获取门户验证码")
    @GetMapping("/captchaImage")
    public AjaxResult getCode(HttpServletResponse response) throws IOException {

        return captchaUtils.getCaptchaData();

    }


    /**
     * 登录方法
     *
     * @param loginBody 登录信息
     * @return 结果
     */
    @ApiOperation("门户登录")
    @PostMapping("/login")
    public AjaxResult login(@RequestBody LoginBody loginBody) throws InvalidCipherTextException {
        String ipAddress = IpUtils.getIpAddr(ServletUtils.getRequest());


        // 构建Redis的key
        String loginKey = "login:" + ipAddress;
        String banKey = "ban:" + ipAddress;

        // 获取当前IP地址的登陆次数
        String loginAttemptsStr = redisUtil.getStr(loginKey);

        // 如果没有登陆记录，初始化次数为1
        int loginAttempts = (loginAttemptsStr != null) ? Integer.parseInt(loginAttemptsStr) : 0;

        // 如果超过最大登陆次数，触发封禁
        if (loginAttempts >= MAX_LOGIN_ATTEMPTS) {
            throw new ServiceException("IP地址已被封禁，请稍后再试。");
        }

        // 增加登陆次数并更新超时时间
        loginAttempts++;
        redisUtil.setStr(loginKey, String.valueOf(loginAttempts), BAN_TIME);

        String decrypt = Sm2Utils.decrypt(loginBody.getPassword(), privateKeyHex);
        loginBody.setPassword(decrypt);
        // 此处执行实际的登录逻辑
        sysLoginService.validateCaptcha(loginBody.getUsername(), loginBody.getCode(), loginBody.getUuid());
        CbCompanyUser user = cbCompanyUserService.selectCbCompanyUserByuserName(loginBody.getUsername());
        if (ObjectUtil.isNull(user)) {
            throw new ServiceException("登录用户：" + loginBody.getUsername() + " 不存在");
        }
        if (!SecurityUtils.matchesPassword(loginBody.getPassword(), user.getPassword())) {
            throw new ServiceException("用户名密码错误!");
        }
        Map<String, Object> claims = new HashMap<>();
        String uuid = IdUtils.fastUUID();
        claims.put(Constants.PORTAL_USER_KEY, uuid);
        String token = tokenService.createTokenStr(claims);
        redisCache.setCacheObject(Constants.PORTAL_USER_KEY + uuid, JsonUtils.toJson(user), 120, TimeUnit.MINUTES);
        AjaxResult ajax = AjaxResult.success();
        ajax.put(Constants.TOKEN, token);
        user.setLoginDate(DateUtils.getNowDate());
        user.setLoginIp(ipAddress);
        cbCompanyUserService.updateById(user);
        ajax.put(Constants.PORTAL_USER_KEY, user);
        return ajax;


    }

    /**
     * 获取公密接口
     */
    @PostMapping("/genPUPRkey")
    public AjaxResult genPUPRkey() throws NoSuchAlgorithmException {
        Sm2 sm2 = Sm2Utils.genPUPRkey();
        privateKeyHex = sm2.getPrivateKeyHex();
        return AjaxResult.success(sm2.getPublicKeyHex());
    }

    /**
     * 用户注册
     *
     * @param user
     * @return
     */
    @ApiOperation("门户注册")
    @PostMapping("/register")
    public AjaxResult register(@RequestBody CbCompanyUserVo user) {
        String username = user.getUserName(), password = user.getPassword();
        sysLoginService.validateCaptcha(username, user.getCode(), user.getUuid());
        String regex = "^(?=.*\\d)(?=.*[a-zA-Z])[a-zA-Z0-9]+$";
        Pattern r = Pattern.compile(regex);
        if (StringUtils.isNull(username)) {
            Assert.isTrue(false, "用户名不能为空");
        } else if (StringUtils.isNull(password)) {
            Assert.isTrue(false, "密码不能为空");
        } else if (!r.matcher(password).matches()){
            Assert.isTrue(false,"密码必须字母+数字");
        }else if (0 < cbCompanyUserService.checkCbCompanyUserByuserName(username)) {
            Assert.isTrue(false, "注册用户'" + username + "'失败，账号已存在");
        }
        CbCompanyUser companyUser = BeanUtil.copyProperties(user, CbCompanyUser.class);
        companyUser.setUserName(username);
        companyUser.setNickName(user.getNickName());
        companyUser.setPassword(SecurityUtils.encryptPassword(password));
        companyUser.setLoginDate(DateUtils.getNowDate());
        companyUser.setLoginIp(IpUtils.getIpAddr(ServletUtils.getRequest()));
        boolean save = cbCompanyUserService.save(companyUser);
        if (SPECIALIST.equals(user.getType())&&save){
            CbModifyCompanyUser cbModifyCompanyUser = BeanUtil.copyProperties(companyUser, CbModifyCompanyUser.class);
            cbModifyCompanyUser.setAudit(1);
            cbModifyCompanyUserService.save(cbModifyCompanyUser);
            wfInstActionTaskService.statrWfqueue(WorkFlowServiceEnums.COMPANYUSERSERVICE.getService(), companyUser.getId(), companyUser.getUserName());
        }
        return AjaxResult.success("注册成功");
    }

    /**
     * 找回密码
     * @param user
     * @return
     */
    @ApiOperation("门户找回密码")
    @PostMapping("/findPAS")
    public AjaxResult findPAS(@RequestBody RegisterBody user) {
        String username = user.getUsername(), password = user.getPassword();
        sysLoginService.validateCaptcha(username, user.getCode(), user.getUuid());
        CbCompanyUser companyUser = cbCompanyUserService.selectCbCompanyUserByuserName(username);
        if (StringUtils.isEmpty(username)) {
            Assert.isTrue(false, "用户名不能为空");
        } else if (ObjectUtil.isNull(companyUser)) {
            Assert.isTrue(false, "用户不存在！");
        }
        companyUser.setPassword(SecurityUtils.encryptPassword(password));
        cbCompanyUserService.updateById(companyUser);
        return AjaxResult.success("修改成功");
    }


}
