package com.hjy.module.EasyExcel;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.excel.metadata.CellExtra;
import com.alibaba.excel.metadata.data.ReadCellData;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.util.ListUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.spring.SpringUtils;
import com.hjy.module.service.gather.impl.CbGatherServiceImpl;
import com.hjy.module.vo.gather.GatherImportVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 海关报表
 */
@Slf4j
public class GatherDataCustomsListener implements ReadListener<GatherImportVo> {
    /**
     * 每隔5条存储数据库，实际使用中可以100条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 100;

    /**
     *
     * 数据类型(0=量化指标数据,1=月报表数据)
     */
    private String type;

    /**
     * 缓存的数据
     */
    private List<GatherImportVo> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

    private List<String> headLists = Lists.newArrayList();
    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private CbGatherServiceImpl gatherService;

    private List<Integer> mergeList = Lists.newArrayList();

    public GatherDataCustomsListener(String type) {
        this.type = type;
        gatherService = SpringUtils.getBean(CbGatherServiceImpl.class);
    }


    /**
     * 这个每一条数据解析都会来调用
     *
     * @param data    one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    @Override
    public void invoke(GatherImportVo data, AnalysisContext context) {
        cachedDataList.add(data);
    }

    /**
     * 在转换异常 获取其他异常下会调用本接口。抛出异常则停止读取。如果这里不抛出异常则 继续读取下一行。
     *
     * @param exception
     * @param context
     * @throws Exception
     */
    @Override
    public void onException(Exception exception, AnalysisContext context) {
        log.error("解析失败，但是继续解析下一行:{}", exception.getMessage());
        // 如果是某一个单元格的转换异常 能获取到具体行号
        // 如果要获取头的信息 配合invokeHeadMap使用
        if (exception instanceof ExcelDataConvertException) {
            ExcelDataConvertException excelDataConvertException = (ExcelDataConvertException)exception;
            log.error("第{}行，第{}列解析异常，数据为:{}", excelDataConvertException.getRowIndex(),
                    excelDataConvertException.getColumnIndex(), excelDataConvertException.getCellData());
        }
    }

    /**
     * 这里会一行行的返回头
     *
     * @param headMap
     * @param context
     */
    @Override
    public void invokeHead(Map<Integer, ReadCellData<?>> headMap, AnalysisContext context) {
        try {
            String str = headMap.get(3).getStringValue();
            if (StringUtils.isNotNull(str))headLists.add(str);
        }catch (Exception e){
            Assert.isTrue(false,"第四列的保税区信息读取异常，请检查文件");
        }

    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 校验数据完整姓
        cachedDataList.forEach(item ->{
            if (StringUtils.isNull(item.getNormName()) && type.equals("0")){
                Assert.isTrue(false,"指标分类不可为空，请检查文件");
            }
            if (StringUtils.isNull(item.getNormfeeName())){
                Assert.isTrue(false,"指标名称不可为空，请检查文件");
            }
            if (StringUtils.isNull(item.getCollCalue())){
                Assert.isTrue(false,"录入数据不可为空，请检查文件");
            }
        });
        if (CollectionUtil.isEmpty(headLists)){
            Assert.isTrue(false,"第四列的保税区信息及月份不能为空，请检查文件");
        }
        // 这里开始处理数据
        saveData();

    }

    /**
     * 处理实际数据 业务代码
     */
    private void saveData() {
        gatherService.saveImportCustomsData(headLists,cachedDataList,type);
        log.info("{}列头数据！", headLists.size());
        log.info("{}条数据，开始存储数据库！", cachedDataList.size());
        log.info("存储数据库成功！");
    }

    /**
     * 获取合并单元值
     * @param extra
     * @param context
     */
    @Override
    public void extra(CellExtra extra, AnalysisContext context) {
        switch (extra.getType()) {
            case MERGE: // 额外信息是合并单元格
                if (extra.getRowIndex() >= 2 && type.equals("0")) {
                    int firstRowIndex = extra.getFirstRowIndex() - 2;
                    int lastRowIndex = extra.getLastRowIndex() - 2;
                    this.copyNormName(firstRowIndex,lastRowIndex);
                }
                break;
            default: break;
        }
    }

    /**
     * 合并单元值业务处理
     * @param firstRowIndex 合并第一行
     * @param lastRowIndex 合并最后一行
     */
    public void copyNormName(int firstRowIndex, int lastRowIndex){
        AtomicInteger index = new AtomicInteger();
        AtomicReference<String> normName= new AtomicReference<>("");
        cachedDataList.stream().forEach(vo ->{
            if(index.get()==firstRowIndex){
                normName.set(vo.getNormName());
            }
            if(index.get()>firstRowIndex&&index.get()<=lastRowIndex){
                vo.setNormName(String.valueOf(normName));
            }
            index.getAndIncrement();
        });
    }


}
