package com.hjy.module.aspect;

import cn.hutool.json.JSONException;
import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hjy.common.annotation.PageDict;
import com.hjy.common.constant.Constants;
import com.hjy.common.core.domain.entity.SysDictModel;
import com.hjy.common.core.page.TableDataInfo;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.reflect.ReflectUtils;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.*;

@Aspect
@Component
@Slf4j
public class PageDictAspect {


    @Lazy
    @Autowired
    private DictFilterUtils dictFilterUtils;

    @Autowired
    public RedisTemplate redisTemplate;

    @Autowired
    private ObjectMapper objectMapper;

    /**
     * 定义切点Pointcut
     */
    @Pointcut("execution(public * com.hjy..*.*Controller.*(..))")
    public void excudeService() {
    }

    @Around("excudeService()")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
        Object result = null;
        try {
            result = pjp.proceed();
            result=this.parseDictText(result);
        }catch (JSONException | JsonProcessingException | com.alibaba.fastjson2.JSONException e){
            log.error("json解析失败:" + e.getMessage());
            return result;
        }
        return result;

    }


    /**
     * 本方法针对返回对象为TableDataInfo 的分页列表数据进行动态字典注入
     * 字典注入实现 通过对实体类添加注解@PageDict 来标识需要的字典内容,字典分为单字典code即可 ，
     * 示例为SysUser   字段为sex 添加了注解@PageDict(dicCode = "sex") 会在字典服务立马查出来对应的text 然后在请求list的时候将这个字典text，已字段名称加_dictText形式返回到前端
     * 例输入当前返回值的就会多出一个sex_dictText字段
     * @param result
     */
    private Object parseDictText(Object result) throws NoSuchFieldException {
        if (result instanceof TableDataInfo) {

                List<JSONObject> items = new ArrayList<>();
                //step.1 筛选出加了 Dict 注解的字段列表
                List<Field> dictFieldList = new ArrayList<>();
                // 字典数据列表， key = 字典code，value=数据列表
                Map<String, List<String>> dataListMap = new HashMap<>(5);
                //取出结果集
                List<Object> records= (List<Object>) ((TableDataInfo<?>) result).getRows();;
                Boolean hasDict= dictFilterUtils.checkHasDict(records,PageDict.class);
                if(!hasDict){
                    return result;
                }
                log.debug(" __ 进入字典翻译切面 DictAspect —— " );
                for (Object record : records) {
                    String json="{}";
                    try {
                        json = objectMapper.writeValueAsString(record);
                    } catch (JsonProcessingException e) {
                        log.error("json解析失败"+e.getMessage(),e);
                    }
                    JSONObject item = JSONObject.parseObject(json);
                    // 遍历所有字段，把字典Code取出来，放到 map 里
                    for (Field field : ReflectUtils.getAllFields(record)) {
                        String value = item.getString(field.getName());
                        if (StringUtils.isEmpty(value)) {
                            continue;
                        }

                        if (AnnotationUtils.findAnnotation(field,PageDict.class) != null) {
                            if (!dictFieldList.contains(field)) {
                                dictFieldList.add(field);
                            }
                            String code = field.getAnnotation(PageDict.class).dicCode();
                            String text = field.getAnnotation(PageDict.class).dicText();
                            String table = field.getAnnotation(PageDict.class).dictTable();

                            List<String> dataList;
                            String dictCode = code;
                            if (!StringUtils.isEmpty(table)) {
                                dictCode = String.format("%s,%s,%s", table, text, code);
                            }
                            dataList = dataListMap.computeIfAbsent(dictCode, k -> new ArrayList<>());
                            dictFilterUtils.listAddAllDeduplicate(dataList, Arrays.asList(value.split(",")));
                        }
                    }
                    items.add(item);
                }

                //step.2 调用翻译方法，一次性翻译
                Map<String, List<SysDictModel>> translText = dictFilterUtils.translateAllDict(dataListMap);

                //step.3 将翻译结果填充到返回结果里
                for (JSONObject record : items) {
                    for (Field field : dictFieldList) {
                        String code = field.getAnnotation(PageDict.class).dicCode();
                        String text = field.getAnnotation(PageDict.class).dicText();
                        String table = field.getAnnotation(PageDict.class).dictTable();

                        String fieldDictCode = code;
                        if (!StringUtils.isEmpty(table)) {
                            fieldDictCode = String.format("%s,%s,%s", table, text, code);
                        }

                        String value = record.getString(field.getName());
                        if (StringUtils.isNotEmpty(value)) {
                            List<SysDictModel> dictModels = translText.get(fieldDictCode);
                            if(dictModels==null || dictModels.size()==0){
                                continue;
                            }

                            String textValue = dictFilterUtils.translDictText(dictModels, value);
                            record.put(field.getName() + Constants.DICT_TEXT_SUFFIX, textValue);
                        }
                    }
                }
                ((TableDataInfo<JSONObject>) result).setRows(items);
        }
        return result;
    }



}
