package com.hjy.system.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.hjy.common.annotation.DataScope;
import com.hjy.common.constant.UserConstants;
import com.hjy.common.core.domain.TreeSelect;
import com.hjy.common.core.domain.entity.SysDept;
import com.hjy.common.core.domain.entity.SysRole;
import com.hjy.common.core.domain.entity.SysUser;
import com.hjy.common.core.domain.model.LoginUser;
import com.hjy.common.core.text.Convert;
import com.hjy.common.exception.ServiceException;
import com.hjy.common.utils.SecurityUtils;
import com.hjy.common.utils.StringUtils;
import com.hjy.common.utils.spring.SpringUtils;
import com.hjy.system.mapper.SysDeptMapper;
import com.hjy.system.mapper.SysRoleMapper;
import com.hjy.system.service.ISysDeptService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 部门管理 服务实现
 *
 * @author HJY
 */
@Service
@Slf4j
public class SysDeptServiceImpl implements ISysDeptService {
    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysDept> selectDeptList(SysDept dept) {
        List<SysDept> deptList = deptMapper.selectDeptList(dept);
        return deptList;
    }

    /**
     * 查询部门树结构信息
     *
     * @param dept 部门信息
     * @return 部门树信息集合
     */
    @Override
    public List<TreeSelect> selectDeptTreeList(SysDept dept) {
//        SysDeptServiceImpl aopProxy = SpringUtils.getAopProxy(this);
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String username = loginUser.getUsername();
        Long deptId = loginUser.getDeptId();
        List<SysDept> depts = this.selectDeptList(dept);
        log.info("部门列表：", depts);
        return buildDeptTreeSelect(depts);
    }

    /**
     * 构建前端所需要树结构
     *
     * @param depts 部门列表
     * @return 树结构列表
     */
    @Override
    public List<SysDept> buildDeptTree(List<SysDept> depts) {
        List<SysDept> returnList = new ArrayList<SysDept>();
        List<Long> tempList = new ArrayList<Long>();
        for (SysDept dept : depts) {
            tempList.add(dept.getDeptId());
        }
        for (SysDept dept : depts) {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(dept.getParentId())) {
                recursionFn(depts, dept);
                returnList.add(dept);
            }
        }
        if (returnList.isEmpty()) {
            returnList = depts;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param depts 部门列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildDeptTreeSelect(List<SysDept> depts) {
        List<SysDept> deptTrees = buildDeptTree(depts);
        return deptTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @return 选中部门列表
     */
    @Override
    public List<Long> selectDeptListByRoleId(Long roleId) {
        SysRole role = roleMapper.selectRoleById(roleId);
        return deptMapper.selectDeptListByRoleId(roleId, role.isDeptCheckStrictly());
    }

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
    @Override
    public SysDept selectDeptById(Long deptId) {
        return deptMapper.selectById(deptId);
    }

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
    @Override
    public int selectNormalChildrenDeptById(Long deptId) {
        return deptMapper.selectNormalChildrenDeptById(deptId);
    }

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public boolean hasChildByDeptId(Long deptId) {
        int result = deptMapper.hasChildByDeptId(deptId);
        return result > 0;
    }

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果 true 存在 false 不存在
     */
    @Override
    public boolean checkDeptExistUser(Long deptId) {
        int result = deptMapper.checkDeptExistUser(deptId);
        return result > 0;
    }

    /**
     * 校验部门名称是否唯一
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public String checkDeptNameUnique(SysDept dept) {
        Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
        SysDept info = deptMapper.checkDeptNameUnique(dept.getDeptName(), dept.getParentId());
        if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验部门是否有数据权限
     *
     * @param deptId 部门id
     */
    @Override
    public void checkDeptDataScope(Long deptId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            SysDept dept = new SysDept();
            dept.setDeptId(deptId);
            List<SysDept> depts = SpringUtils.getAopProxy(this).selectDeptList(dept);
            if (StringUtils.isEmpty(depts)) {
                throw new ServiceException("没有权限访问部门数据！");
            }
        }
    }

    /**
     * 新增保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int insertDept(SysDept dept) {
        SysDept info = deptMapper.selectDeptById(dept.getParentId());
        // 如果父节点不为正常状态,则不允许新增子节点
        if (!UserConstants.DEPT_NORMAL.equals(info.getStatus())) {
            throw new ServiceException("部门停用，不允许新增");
        }
        dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
        if (dept.getUseArea() != null) {
            //计算部门面积使用率  使用率=已使用面/(仓库+冷库+油库+厂房)*100
            double v = dept.getUseArea() / (((dept.getWarehouse() / 10000) + dept.getRefrigeratory() + dept.getOilDepot() + dept.getWorkshop()) * 100);
            if (v == 0.0d) {
                dept.setUsageRate("0%");
            } else {
                dept.setUsageRate(String.format("%.2f", v) + "%");
            }
        }
        return this.deptMapper.insertDept(dept);
    }

    /**
     * 修改保存部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
    @Override
    public int updateDept(SysDept dept) {
        // 获取父部门信息
        SysDept newParentDept = deptMapper.selectDeptById(dept.getParentId());
        // 获取子部门信息
        SysDept oldDept = deptMapper.selectDeptById(dept.getDeptId());
        if (StringUtils.isNull(newParentDept)){
            // 如果为空 则自己就是父部门
            dept.setType(0);
        }
        if (StringUtils.isNotNull(newParentDept) && StringUtils.isNotNull(oldDept)) {
            String newAncestors = newParentDept.getAncestors() + "," + newParentDept.getDeptId();
            String oldAncestors = oldDept.getAncestors();
            dept.setAncestors(newAncestors);
            updateDeptChildren(dept.getDeptId(), newAncestors, oldAncestors);
            dept.setType(newParentDept.getType() + 1);
        }

        if (Objects.nonNull(dept.getUseArea()) && Objects.nonNull(dept.getWarehouse()) && Objects.nonNull(dept.getRefrigeratory()) &&
                Objects.nonNull(dept.getOilDepot()) && Objects.nonNull(dept.getWorkshop())) {
            //计算部门面积使用率  使用率=已使用面/(仓库+冷库+油库+厂房)*100
            double v = dept.getUseArea() / (((dept.getWarehouse() / 10000) + dept.getRefrigeratory() + dept.getOilDepot() + dept.getWorkshop()) * 100);
            if (v == 0.0d) {
                dept.setUsageRate("0%");
            } else {
                dept.setUsageRate(String.format("%.2f", v) + "%");
            }
        }

        int result = deptMapper.updateDept(dept);
        if (UserConstants.DEPT_NORMAL.equals(dept.getStatus()) && StringUtils.isNotEmpty(dept.getAncestors())
                && !StringUtils.equals("0", dept.getAncestors())) {
            // 如果该部门是启用状态，则启用该部门的所有上级部门
            updateParentDeptStatusNormal(dept);
        }
        return result;
    }

    /**
     * 修改该部门的父级部门状态
     *
     * @param dept 当前部门
     */
    private void updateParentDeptStatusNormal(SysDept dept) {
        String ancestors = dept.getAncestors();
        Long[] deptIds = Convert.toLongArray(ancestors);
        deptMapper.updateDeptStatusNormal(deptIds);
    }

    /**
     * 修改子元素关系
     *
     * @param deptId       被修改的部门ID
     * @param newAncestors 新的父ID集合
     * @param oldAncestors 旧的父ID集合
     */
    public void updateDeptChildren(Long deptId, String newAncestors, String oldAncestors) {
        List<SysDept> children = deptMapper.selectChildrenDeptById(deptId);
        for (SysDept child : children) {
            child.setAncestors(child.getAncestors().replaceFirst(oldAncestors, newAncestors));
        }
        if (children.size() > 0) {
            deptMapper.updateDeptChildren(children);
        }
    }

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
    @Override
    public int deleteDeptById(Long deptId) {
        return deptMapper.deleteDeptById(deptId);
    }

    /**
     * 根据名称查询
     *
     * @param deptNamee
     * @return
     */
    @Override
    public SysDept selectDeptByName(String deptNamee) {
        SysDept dept = new SysDept();
        dept.setDeptName(deptNamee);
        List<SysDept> deptList = deptMapper.selectDeptList(dept);
        if (CollectionUtil.isEmpty(deptList)) {
            Assert.isTrue(false, "未找到对应保税区");
        }
        if (deptList.size() > 1) {
//            Assert.isTrue(false,"错误，找到多个保税区，无法指定");
        }
        return deptList.get(0);
    }

    /**
     * 根据邮箱查询
     *
     * @param email
     * @return
     */
    @Override
    public SysDept selectDeptByEmail(String email) {
        SysDept dept = new SysDept();
        dept.setEmail(email);
        List<SysDept> deptList = deptMapper.selectDeptList(dept);
        if (CollectionUtil.isEmpty(deptList)) {
            Assert.isTrue(false, "未找到对应保税区");
        }
        if (deptList.size() > 1) {
//            Assert.isTrue(false,"错误，找到多个保税区，无法指定");
        }
        return deptList.get(0);
    }


    /**
     * 递归列表
     */
    private void recursionFn(List<SysDept> list, SysDept t) {
        // 得到子节点列表
        List<SysDept> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysDept tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysDept> getChildList(List<SysDept> list, SysDept t) {
        List<SysDept> tlist = new ArrayList<SysDept>();
        Iterator<SysDept> it = list.iterator();
        while (it.hasNext()) {
            SysDept n = it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getDeptId().longValue()) {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysDept> list, SysDept t) {
        return getChildList(list, t).size() > 0;
    }

    @Override
    public boolean checkDeptStatus(Long deptId) {
        SysDept sysDept = deptMapper.selectDeptById(deptId);
        if (null == sysDept) {
            return false;
        }

        if (sysDept.getStatus().equals("0")) {
            return true;
        }

        return false;
    }

    @Override
    public Long getCbidByuserId(String userId) {
        SysDept sysDept = deptMapper.getDeptByuserId(userId);
        if (sysDept.getType() == 3) {
            return sysDept.getParentId();
        }
        return sysDept.getDeptId();
    }

    @Override
    public void update(SysDept dept) {
        deptMapper.edit(dept);
    }

    @Override
    public SysDept getDeptOrderSumMax(String deptId) {
        SysDept dept = deptMapper.getDeptOrderSumMax(deptId);
        return dept;
    }

    @Override
    public void deleteBydeptNameAndPraentId(String deptName, String parentId){
        deptMapper.deleteBydeptNameAndPraentId(deptName,parentId);
    }
}
